@@ -34,6 +34,7 @@
 #include "EditingStyle.h"
 #include "EditorInsertAction.h"
 #include "FindOptions.h"
+#include "FrameDestructionObserver.h"
 #include "FrameSelection.h"
 #include "TextChecking.h"
 #include "VisibleSelection.h"
@@ -81,7 +82,7 @@ struct CompositionUnderline {
 
 enum EditorCommandSource { CommandFromMenuOrKeyBinding, CommandFromDOM, CommandFromDOMWithUserInterface };
 
-class Editor {
+class Editor : public FrameDestructionObserver {
 public:
     Editor(Frame*);
     ~Editor();
@@ -384,7 +385,8 @@ public:
     void deviceScaleFactorChanged();
 
 private:
-    Frame* m_frame;
+    virtual void willDetachPage() OVERRIDE;
+
     OwnPtr<DeleteButtonController> m_deleteButtonController;
     RefPtr<CompositeEditCommand> m_lastEditCommand;
     RefPtr<Node> m_removedAnchor;
