@@ -105,12 +105,12 @@ HTMLCanvasElement::~HTMLCanvasElement()
     m_context.clear(); // Ensure this goes away before the ImageBuffer.
 }
 
-void HTMLCanvasElement::parseMappedAttribute(Attribute* attr)
+void HTMLCanvasElement::parseAttribute(Attribute* attr)
 {
     const QualifiedName& attrName = attr->name();
     if (attrName == widthAttr || attrName == heightAttr)
         reset();
-    HTMLElement::parseMappedAttribute(attr);
+    HTMLElement::parseAttribute(attr);
 }
 
 RenderObject* HTMLCanvasElement::createRenderer(RenderArena* arena, RenderStyle* style)
@@ -177,7 +177,7 @@ CanvasRenderingContext* HTMLCanvasElement::getContext(const String& type, Canvas
 #if ENABLE(WEBGL)    
     Settings* settings = document()->settings();
     if (settings && settings->webGLEnabled()
-#if !PLATFORM(CHROMIUM) && !PLATFORM(GTK)
+#if !PLATFORM(CHROMIUM) && !PLATFORM(GTK) && !PLATFORM(EFL)
         && settings->acceleratedCompositingEnabled()
 #endif
         ) {
@@ -205,7 +205,7 @@ CanvasRenderingContext* HTMLCanvasElement::getContext(const String& type, Canvas
 
 void HTMLCanvasElement::didDraw(const FloatRect& rect)
 {
-    m_copiedImage.clear(); // Clear our image snapshot if we have one.
+    clearCopiedImage();
 
     if (RenderBox* ro = renderBox()) {
         FloatRect destRect = ro->contentBoxRect();
@@ -282,9 +282,9 @@ void HTMLCanvasElement::paint(GraphicsContext* context, const LayoutRect& r, boo
         ImageBuffer* imageBuffer = buffer();
         if (imageBuffer) {
             if (m_presentedImage)
-                context->drawImage(m_presentedImage.get(), ColorSpaceDeviceRGB, r, CompositeSourceOver, useLowQualityScale);
+                context->drawImage(m_presentedImage.get(), ColorSpaceDeviceRGB, pixelSnappedIntRect(r), CompositeSourceOver, useLowQualityScale);
             else
-                context->drawImageBuffer(imageBuffer, ColorSpaceDeviceRGB, r, CompositeSourceOver, useLowQualityScale);
+                context->drawImageBuffer(imageBuffer, ColorSpaceDeviceRGB, pixelSnappedIntRect(r), CompositeSourceOver, useLowQualityScale);
         }
     }
 
@@ -325,7 +325,18 @@ void HTMLCanvasElement::setSurfaceSize(const IntSize& size)
     m_size = size;
     m_hasCreatedImageBuffer = false;
     m_imageBuffer.clear();
-    m_copiedImage.clear();
+    clearCopiedImage();
+}
+
+String HTMLCanvasElement::toEncodingMimeType(const String& mimeType)
+{
+    String lowercaseMimeType = mimeType.lower();
+
+    // FIXME: Make isSupportedImageMIMETypeForEncoding threadsafe (to allow this method to be used on a worker thread).
+    if (mimeType.isNull() || !MIMETypeRegistry::isSupportedImageMIMETypeForEncoding(lowercaseMimeType))
+        lowercaseMimeType = "image/png";
+
+    return lowercaseMimeType;
 }
 
 String HTMLCanvasElement::toDataURL(const String& mimeType, const double* quality, ExceptionCode& ec)
@@ -338,23 +349,19 @@ String HTMLCanvasElement::toDataURL(const String& mimeType, const double* qualit
     if (m_size.isEmpty() || !buffer())
         return String("data:,");
 
-    String lowercaseMimeType = mimeType.lower();
-
-    // FIXME: Make isSupportedImageMIMETypeForEncoding threadsafe (to allow this method to be used on a worker thread).
-    if (mimeType.isNull() || !MIMETypeRegistry::isSupportedImageMIMETypeForEncoding(lowercaseMimeType))
-        lowercaseMimeType = "image/png";
+    String encodingMimeType = toEncodingMimeType(mimeType);
 
 #if USE(CG) || USE(SKIA)
     // Try to get ImageData first, as that may avoid lossy conversions.
     RefPtr<ImageData> imageData = getImageData();
 
     if (imageData)
-        return ImageDataToDataURL(*imageData, lowercaseMimeType, quality);
+        return ImageDataToDataURL(*imageData, encodingMimeType, quality);
 #endif
 
     makeRenderingResultsAvailable();
-      
-    return buffer()->toDataURL(lowercaseMimeType, quality);
+
+    return buffer()->toDataURL(encodingMimeType, quality);
 }
 
 PassRefPtr<ImageData> HTMLCanvasElement::getImageData()
@@ -395,6 +402,13 @@ FloatSize HTMLCanvasElement::convertLogicalToDevice(const FloatSize& logicalSize
     return FloatSize(width, height);
 }
 
+FloatSize HTMLCanvasElement::convertDeviceToLogical(const FloatSize& deviceSize) const
+{
+    float width = ceilf(deviceSize.width() / m_deviceScaleFactor);
+    float height = ceilf(deviceSize.height() / m_deviceScaleFactor);
+    return FloatSize(width, height);
+}
+
 SecurityOrigin* HTMLCanvasElement::securityOrigin() const
 {
     return document()->securityOrigin();
@@ -409,13 +423,13 @@ bool HTMLCanvasElement::shouldAccelerate(const IntSize& size) const
 {
 #if USE(IOSURFACE_CANVAS_BACKING_STORE)
     UNUSED_PARAM(size);
-    return document()->settings()->canvasUsesAcceleratedDrawing();
+    return document()->settings() && document()->settings()->canvasUsesAcceleratedDrawing();
 #elif ENABLE(ACCELERATED_2D_CANVAS)
     if (m_context && !m_context->is2d())
         return false;
 
     Settings* settings = document()->settings();
-    if (!settings->accelerated2dCanvasEnabled())
+    if (!settings || !settings->accelerated2dCanvasEnabled())
         return false;
 
     // Do not use acceleration for small canvas.
@@ -429,6 +443,22 @@ bool HTMLCanvasElement::shouldAccelerate(const IntSize& size) const
 #endif
 }
 
+bool HTMLCanvasElement::shouldDefer() const
+{
+#if USE(SKIA)
+    if (m_context && !m_context->is2d())
+        return false;
+
+    Settings* settings = document()->settings();
+    if (!settings || !settings->deferred2dCanvasEnabled())
+        return false;
+
+    return true;
+#else
+    return false;
+#endif
+}
+
 void HTMLCanvasElement::createImageBuffer() const
 {
     ASSERT(!m_imageBuffer);
@@ -451,13 +481,20 @@ void HTMLCanvasElement::createImageBuffer() const
     if (!bufferSize.width() || !bufferSize.height())
         return;
 
-    RenderingMode renderingMode = shouldAccelerate(bufferSize) ? Accelerated : Unaccelerated;
-    m_imageBuffer = ImageBuffer::create(bufferSize, ColorSpaceDeviceRGB, renderingMode);
+    RenderingMode renderingMode = shouldAccelerate(bufferSize) ? Accelerated : 
+#if USE(SKIA)
+        UnacceleratedNonPlatformBuffer;
+#else
+        Unaccelerated;
+#endif
+    DeferralMode deferralMode = shouldDefer() ? Deferred : NonDeferred;
+    m_imageBuffer = ImageBuffer::create(bufferSize, ColorSpaceDeviceRGB, renderingMode, deferralMode);
     if (!m_imageBuffer)
         return;
     m_imageBuffer->context()->scale(FloatSize(bufferSize.width() / logicalSize.width(), bufferSize.height() / logicalSize.height()));
     m_imageBuffer->context()->setShadowsIgnoreTransforms(true);
     m_imageBuffer->context()->setImageInterpolationQuality(DefaultInterpolationQuality);
+    m_imageBuffer->context()->setStrokeThickness(1);
 
 #if USE(JSC)
     JSC::JSLock lock(JSC::SilenceAssertionsOnly);
