@@ -175,6 +175,15 @@ void Geolocation::Watchers::set(int id, PassRefPtr<GeoNotifier> prpNotifier)
     m_notifierToIdMap.set(notifier.release(), id);
 }
 
+Geolocation::GeoNotifier* Geolocation::Watchers::find(int id)
+{
+    ASSERT(id > 0);
+    IdToNotifierMap::iterator iter = m_idToNotifierMap.find(id);
+    if (iter == m_idToNotifierMap.end())
+        return 0;
+    return iter->second.get();
+}
+
 void Geolocation::Watchers::remove(int id)
 {
     ASSERT(id > 0);
@@ -216,7 +225,7 @@ void Geolocation::Watchers::getNotifiersVector(GeoNotifierVector& copy) const
 }
 
 Geolocation::Geolocation(Frame* frame)
-    : m_frame(frame)
+    : DOMWindowProperty(frame)
 #if !ENABLE(CLIENT_BASED_GEOLOCATION)
     , m_service(GeolocationService::create(this))
 #endif
@@ -246,13 +255,16 @@ void Geolocation::reset()
 #if ENABLE(CLIENT_BASED_GEOLOCATION)
         page->geolocationController()->cancelPermissionRequest(this);
 #else
-        page->chrome()->cancelGeolocationPermissionRequestForFrame(m_frame, this);
+        page->chrome()->client()->cancelGeolocationPermissionRequestForFrame(m_frame, this);
 #endif
     }
     // The frame may be moving to a new page and we want to get the permissions from the new page's client.
     m_allowGeolocation = Unknown;
     cancelAllRequests();
     stopUpdating();
+#if USE(PREEMPT_GEOLOCATION_PERMISSION)
+    m_pendingForPermissionNotifiers.clear();
+#endif
 }
 
 void Geolocation::disconnectFrame()
@@ -261,7 +273,7 @@ void Geolocation::disconnectFrame()
     reset();
     if (m_frame && m_frame->document())
         m_frame->document()->setUsingGeolocation(false);
-    m_frame = 0;
+    DOMWindowProperty::disconnectFrame();
 }
 
 Geoposition* Geolocation::lastPosition()
@@ -418,6 +430,10 @@ void Geolocation::clearWatch(int watchId)
     if (watchId < firstAvailableWatchId)
         return;
 
+#if USE(PREEMPT_GEOLOCATION_PERMISSION)
+    if (GeoNotifier* notifier = m_watchers.find(watchId))
+        m_pendingForPermissionNotifiers.remove(notifier);
+#endif
     m_watchers.remove(watchId);
     
     if (!hasListeners())
@@ -603,7 +619,7 @@ void Geolocation::requestPermission()
 #if ENABLE(CLIENT_BASED_GEOLOCATION)
     page->geolocationController()->requestPermission(this);
 #else
-    page->chrome()->requestGeolocationPermissionForFrame(m_frame, this);
+    page->chrome()->client()->requestGeolocationPermissionForFrame(m_frame, this);
 #endif
 }
 
@@ -746,7 +762,7 @@ void Geolocation::reset() {}
 
 void Geolocation::disconnectFrame() {}
 
-Geolocation::Geolocation(Frame*) {}
+Geolocation::Geolocation(Frame*) : DOMWindowProperty(0) {}
 
 Geolocation::~Geolocation() {}
 
