@@ -26,14 +26,14 @@
 #include "config.h"
 #include "FrameLoaderClientGtk.h"
 
-#include "AXObjectCache.h"
-#include "AccessibilityObject.h"
 #include "ArchiveResource.h"
 #include "CachedFrame.h"
 #include "Color.h"
 #include "DOMObjectCache.h"
 #include "DocumentLoader.h"
 #include "DocumentLoaderGtk.h"
+#include "ErrorsGtk.h"
+#include "FileSystem.h"
 #include "FormState.h"
 #include "FrameLoader.h"
 #include "FrameNetworkingContextGtk.h"
@@ -65,9 +65,10 @@
 #include "ScriptController.h"
 #include "Settings.h"
 #include "webkiterror.h"
+#include "webkitfavicondatabase.h"
+#include "webkitfavicondatabaseprivate.h"
 #include "webkitglobals.h"
 #include "webkitglobalsprivate.h"
-#include "webkiticondatabase.h"
 #include "webkitnetworkrequest.h"
 #include "webkitnetworkrequestprivate.h"
 #include "webkitnetworkresponse.h"
@@ -123,51 +124,6 @@ String FrameLoaderClient::userAgent(const KURL& url)
     return String::fromUTF8(userAgentString.get());
 }
 
-static void notifyAccessibilityStatus(WebKitWebFrame* frame, WebKitLoadStatus loadStatus)
-{
-    if (loadStatus != WEBKIT_LOAD_PROVISIONAL
-        && loadStatus != WEBKIT_LOAD_FAILED
-        && loadStatus != WEBKIT_LOAD_FINISHED)
-        return;
-
-    WebKitWebFramePrivate* priv = frame->priv;
-    if (!priv->coreFrame || !priv->coreFrame->document())
-        return;
-
-    RenderView* contentRenderer = priv->coreFrame->contentRenderer();
-    if (!contentRenderer)
-        return;
-
-    AXObjectCache* axObjectCache = priv->coreFrame->document()->axObjectCache();
-    if (!axObjectCache)
-        return;
-
-    AccessibilityObject* coreAxObject = axObjectCache->getOrCreate(contentRenderer);
-    if (!coreAxObject)
-        return;
-
-    AtkObject* axObject = coreAxObject->wrapper();
-    if (!axObject || !ATK_IS_DOCUMENT(axObject))
-        return;
-
-    switch (loadStatus) {
-    case WEBKIT_LOAD_PROVISIONAL:
-        g_signal_emit_by_name(axObject, "state-change", "busy", true);
-        if (core(frame)->loader()->loadType() == FrameLoadTypeReload)
-            g_signal_emit_by_name(axObject, "reload");
-        break;
-    case WEBKIT_LOAD_FAILED:
-        g_signal_emit_by_name(axObject, "load-stopped");
-        g_signal_emit_by_name(axObject, "state-change", "busy", false);
-        break;
-    case WEBKIT_LOAD_FINISHED:
-        g_signal_emit_by_name(axObject, "load-complete");
-        g_signal_emit_by_name(axObject, "state-change", "busy", false);
-    default:
-        break;
-    }
-}
-
 static void notifyStatus(WebKitWebFrame* frame, WebKitLoadStatus loadStatus)
 {
     frame->priv->loadStatus = loadStatus;
@@ -177,9 +133,6 @@ static void notifyStatus(WebKitWebFrame* frame, WebKitLoadStatus loadStatus)
     if (frame == webkit_web_view_get_main_frame(webView)) {
         webView->priv->loadStatus = loadStatus;
         g_object_notify(G_OBJECT(webView), "load-status");
-
-        if (AXObjectCache::accessibilityEnabled())
-            notifyAccessibilityStatus(frame, loadStatus);
     }
 }
 
@@ -282,8 +235,9 @@ void FrameLoaderClient::dispatchWillSendRequest(WebCore::DocumentLoader* loader,
         g_free(webResource->priv->uri);
         webResource->priv->uri = g_strdup(request.url().string().utf8().data());
     }
-    
+
     g_signal_emit_by_name(webView, "resource-request-starting", m_frame, webResource, networkRequest.get(), networkResponse.get());
+    g_signal_emit_by_name(m_frame, "resource-request-starting", webResource, networkRequest.get(), networkResponse.get());
 
     // Feed any changes back into the ResourceRequest object.
     SoupMessage* message = webkit_network_request_get_message(networkRequest.get());
@@ -347,13 +301,22 @@ void FrameLoaderClient::frameLoaderDestroyed()
     delete this;
 }
 
-void FrameLoaderClient::dispatchDidReceiveResponse(WebCore::DocumentLoader* loader, unsigned long, const ResourceResponse& response)
+void FrameLoaderClient::dispatchDidReceiveResponse(WebCore::DocumentLoader* loader, unsigned long identifier, const ResourceResponse& response)
 {
     // Update our knowledge of request soup flags - some are only set
     // after the request is done.
     loader->request().setSoupMessageFlags(response.soupMessageFlags());
 
     m_response = response;
+
+    WebKitWebView* webView = getViewFromFrame(m_frame);
+    GOwnPtr<gchar> identifierString(toString(identifier));
+    WebKitWebResource* webResource = webkit_web_view_get_resource(webView, identifierString.get());
+    GRefPtr<WebKitNetworkResponse> networkResponse(adoptGRef(kitNew(response)));
+
+    g_signal_emit_by_name(webResource, "response-received", networkResponse.get());
+    g_signal_emit_by_name(m_frame, "resource-response-received", webResource, networkResponse.get());
+    g_signal_emit_by_name(webView, "resource-response-received", m_frame, webResource, networkResponse.get());
 }
 
 void FrameLoaderClient::dispatchDecidePolicyForResponse(FramePolicyFunction policyFunction, const ResourceResponse& response, const ResourceRequest& resourceRequest)
@@ -521,8 +484,10 @@ PassRefPtr<Widget> FrameLoaderClient::createPlugin(const IntSize& pluginSize, HT
     GtkWidget* gtkWidget = 0;
     g_signal_emit_by_name(getViewFromFrame(m_frame), "create-plugin-widget",
                           mimeTypeString.data(), urlString.data(), hash.get(), &gtkWidget);
-    if (gtkWidget)
+    if (gtkWidget) {
+        gtk_container_add(GTK_CONTAINER(getViewFromFrame(m_frame)), gtkWidget);
         return adoptRef(new GtkPluginWidget(gtkWidget));
+    }
 
     RefPtr<PluginView> pluginView = PluginView::create(core(m_frame), pluginSize, element, url, paramNames, paramValues, mimeType, loadManually);
 
@@ -567,35 +532,6 @@ PassRefPtr<Frame> FrameLoaderClient::createFrame(const KURL& url, const String&
     return childFrame.release();
 }
 
-void FrameLoaderClient::didTransferChildFrameToNewDocument(WebCore::Page*)
-{
-    ASSERT(m_frame);
-
-    // Update the frame's webview to the new parent's webview.
-    Frame* coreFrame = core(m_frame);
-    WebKitWebView* webView = getViewFromFrame(m_frame);
-
-    Frame* parentCoreFrame = coreFrame->tree()->parent();
-    WebKitWebFrame* parentKitFrame = kit(parentCoreFrame);
-    WebKitWebView* parentWebView = getViewFromFrame(parentKitFrame);
-    if (webView != parentWebView)
-        m_frame->priv->webView = parentWebView;
-
-    ASSERT(core(getViewFromFrame(m_frame)) == coreFrame->page());
-}
-
-void FrameLoaderClient::transferLoadingResourceFromPage(WebCore::ResourceLoader* loader, const WebCore::ResourceRequest& request, WebCore::Page* oldPage)
-{
-    ASSERT(oldPage != core(m_frame)->page());
-
-    GOwnPtr<gchar> identifierString(toString(loader->identifier()));
-    ASSERT(!webkit_web_view_get_resource(getViewFromFrame(m_frame), identifierString.get()));
-
-    assignIdentifierToInitialRequest(loader->identifier(), loader->documentLoader(), request);
-
-    webkit_web_view_remove_resource(kit(oldPage), identifierString.get());
-}
-
 void FrameLoaderClient::redirectDataToPlugin(Widget* pluginWidget)
 {
     ASSERT(!m_pluginView);
@@ -631,7 +567,7 @@ void FrameLoaderClient::dispatchDidClearWindowObjectInWorld(DOMWrapperWorld* wor
     ASSERT(coreFrame);
 
     Settings* settings = coreFrame->settings();
-    if (!settings || !settings->isJavaScriptEnabled())
+    if (!settings || !settings->isScriptEnabled())
         return;
 
     // TODO: Consider using g_signal_has_handler_pending() to avoid the overhead
@@ -661,7 +597,7 @@ void FrameLoaderClient::didPerformFirstNavigation() const
 
 void FrameLoaderClient::registerForIconNotification(bool shouldRegister)
 {
-    notImplemented();
+    webkitWebViewRegisterForIconNotification(getViewFromFrame(m_frame), shouldRegister);
 }
 
 void FrameLoaderClient::setMainFrameDocumentReady(bool ready)
@@ -723,6 +659,11 @@ void FrameLoaderClient::didRunInsecureContent(SecurityOrigin*, const KURL&)
     notImplemented();
 }
 
+void FrameLoaderClient::didDetectXSS(const KURL&, bool)
+{
+    notImplemented();
+}
+
 void FrameLoaderClient::makeRepresentation(WebCore::DocumentLoader*)
 {
     m_hasRepresentation = true;
@@ -786,19 +727,41 @@ void FrameLoaderClient::dispatchDidChangeLocationWithinPage()
         g_object_notify(G_OBJECT(webView), "uri");
 }
 
+void FrameLoaderClient::dispatchDidNavigateWithinPage()
+{
+    WebKitWebView* webView = getViewFromFrame(m_frame);
+    WebKitWebFrame* mainFrame = webView->priv->mainFrame;
+    WebKitWebDataSource* dataSource = webkit_web_frame_get_data_source(mainFrame);
+    bool loaderCompleted = !webkit_web_data_source_is_loading(dataSource);
+
+    if (!loaderCompleted)
+        return;
+
+    // No provisional load started, because:
+    // - It will break (no provisional data source at this point).
+    // - There's no provisional load going on anyway, the URI is being
+    //   programatically changed.
+    // FIXME: this is not ideal, but it seems safer than changing our
+    // current contract with the clients about provisional data
+    // sources not being '0' during the provisional load stage.
+    dispatchDidCommitLoad();
+    dispatchDidFinishLoad();
+}
+
 void FrameLoaderClient::dispatchDidPushStateWithinPage()
 {
-    notImplemented();
+    dispatchDidNavigateWithinPage();
 }
 
 void FrameLoaderClient::dispatchDidReplaceStateWithinPage()
 {
-    notImplemented();
+    dispatchDidNavigateWithinPage();
 }
 
 void FrameLoaderClient::dispatchDidPopStateWithinPage()
 {
-    notImplemented();
+    // No need to do anything, we already called
+    // dispatchDidNavigateWithinPage() in PushStateWithinPage().
 }
 
 void FrameLoaderClient::dispatchWillClose()
@@ -811,18 +774,14 @@ void FrameLoaderClient::dispatchDidReceiveIcon()
     if (m_loadingErrorPage)
         return;
 
-    const gchar* frameURI = webkit_web_frame_get_uri(m_frame);
-    WebKitIconDatabase* database = webkit_get_icon_database();
-    g_signal_emit_by_name(database, "icon-loaded", m_frame, frameURI);
-
+    // IconController loads icons only for the main frame.
     WebKitWebView* webView = getViewFromFrame(m_frame);
+    ASSERT(m_frame == webkit_web_view_get_main_frame(webView));
 
-    // Avoid reporting favicons for non-main frames.
-    if (m_frame != webkit_web_view_get_main_frame(webView))
-        return;
-
-    g_object_notify(G_OBJECT(webView), "icon-uri");
-    g_signal_emit_by_name(webView, "icon-loaded", webkit_web_view_get_icon_uri(webView));
+    const char* frameURI = webkit_web_frame_get_uri(m_frame);
+    WebKitFaviconDatabase* database = webkit_get_favicon_database();
+    webkitFaviconDatabaseDispatchDidReceiveIcon(database, frameURI);
+    webkitWebViewIconLoaded(database, frameURI, webView);
 }
 
 void FrameLoaderClient::dispatchDidStartProvisionalLoad()
@@ -1014,7 +973,13 @@ void FrameLoaderClient::setTitle(const StringWithDirection& title, const KURL& u
 
 void FrameLoaderClient::dispatchDidReceiveContentLength(WebCore::DocumentLoader*, unsigned long identifier, int dataLength)
 {
-    notImplemented();
+    WebKitWebView* webView = getViewFromFrame(m_frame);
+    GOwnPtr<gchar> identifierString(toString(identifier));
+    WebKitWebResource* webResource = webkit_web_view_get_resource(webView, identifierString.get());
+
+    g_signal_emit_by_name(webResource, "content-length-received", dataLength);
+    g_signal_emit_by_name(m_frame, "resource-content-length-received", webResource, dataLength);
+    g_signal_emit_by_name(webView, "resource-content-length-received", m_frame, webResource, dataLength);
 }
 
 void FrameLoaderClient::dispatchDidFinishLoading(WebCore::DocumentLoader* loader, unsigned long identifier)
@@ -1043,20 +1008,31 @@ void FrameLoaderClient::dispatchDidFinishLoading(WebCore::DocumentLoader* loader
 
     webkit_web_resource_init_with_core_resource(webResource, coreResource.get());
 
-    // FIXME: This function should notify the application that the resource
-    // finished loading, maybe using a load-status property in the
-    // WebKitWebResource object, similar to what we do for WebKitWebFrame'
-    // signal.
-    notImplemented();
+    g_signal_emit_by_name(webResource, "load-finished");
+    g_signal_emit_by_name(m_frame, "resource-load-finished", webResource);
+    g_signal_emit_by_name(webView, "resource-load-finished", m_frame, webResource);
 }
 
 void FrameLoaderClient::dispatchDidFailLoading(WebCore::DocumentLoader* loader, unsigned long identifier, const ResourceError& error)
 {
     static_cast<WebKit::DocumentLoader*>(loader)->decreaseLoadCount(identifier);
 
-    // FIXME: This function should notify the application that the resource failed
-    // loading, maybe a 'load-error' signal in the WebKitWebResource object.
-    notImplemented();
+    WebKitWebView* webView = getViewFromFrame(m_frame);
+    GOwnPtr<gchar> identifierString(toString(identifier));
+    WebKitWebResource* webResource = webkit_web_view_get_resource(webView, identifierString.get());
+
+    // A NULL WebResource means the load has been interrupted, and
+    // replaced by another one while this resource was being loaded.
+    if (!webResource)
+        return;
+
+    GOwnPtr<GError> webError(g_error_new_literal(g_quark_from_string(error.domain().utf8().data()),
+                                                 error.errorCode(),
+                                                 error.localizedDescription().utf8().data()));
+
+    g_signal_emit_by_name(webResource, "load-failed", webError.get());
+    g_signal_emit_by_name(m_frame, "resource-load-failed", webResource, webError.get());
+    g_signal_emit_by_name(webView, "resource-load-failed", m_frame, webResource, webError.get());
 }
 
 bool FrameLoaderClient::dispatchDidLoadResourceFromMemoryCache(WebCore::DocumentLoader*, const ResourceRequest&, const ResourceResponse&, int length)
@@ -1098,7 +1074,9 @@ void FrameLoaderClient::dispatchDidFailLoad(const ResourceError& error)
 
     String content;
     gchar* fileContent = 0;
-    gchar* errorURI = g_filename_to_uri(DATA_DIR"/webkit-1.0/resources/error.html", NULL, NULL);
+    GOwnPtr<gchar> errorPath(g_build_filename(sharedResourcesPath().data(), "resources", "error.html", NULL));
+    gchar* errorURI = g_filename_to_uri(errorPath.get(), 0, 0);
+
     GFile* errorFile = g_file_new_for_uri(errorURI);
     g_free(errorURI);
 
@@ -1122,7 +1100,7 @@ void FrameLoaderClient::dispatchDidFailLoad(const ResourceError& error)
     g_error_free(webError);
 }
 
-void FrameLoaderClient::download(ResourceHandle* handle, const ResourceRequest& request, const ResourceRequest&, const ResourceResponse& response)
+void FrameLoaderClient::download(ResourceHandle* handle, const ResourceRequest& request, const ResourceResponse& response)
 {
     GRefPtr<WebKitNetworkRequest> networkRequest(adoptGRef(kitNew(request)));
     WebKitWebView* view = getViewFromFrame(m_frame);
@@ -1132,44 +1110,37 @@ void FrameLoaderClient::download(ResourceHandle* handle, const ResourceRequest&
 
 ResourceError FrameLoaderClient::cancelledError(const ResourceRequest& request)
 {
-    return ResourceError(g_quark_to_string(WEBKIT_NETWORK_ERROR), WEBKIT_NETWORK_ERROR_CANCELLED,
-                         request.url().string(), _("Load request cancelled"));
+    return WebCore::cancelledError(request);
 }
 
 ResourceError FrameLoaderClient::blockedError(const ResourceRequest& request)
 {
-    return ResourceError(g_quark_to_string(WEBKIT_POLICY_ERROR), WEBKIT_POLICY_ERROR_CANNOT_USE_RESTRICTED_PORT,
-                         request.url().string(), _("Not allowed to use restricted network port"));
+    return WebCore::blockedError(request);
 }
 
 ResourceError FrameLoaderClient::cannotShowURLError(const ResourceRequest& request)
 {
-    return ResourceError(g_quark_to_string(WEBKIT_POLICY_ERROR), WEBKIT_POLICY_ERROR_CANNOT_SHOW_URL,
-                         request.url().string(), _("URL cannot be shown"));
+    return WebCore::cannotShowURLError(request);
 }
 
 ResourceError FrameLoaderClient::interruptedForPolicyChangeError(const ResourceRequest& request)
 {
-    return ResourceError(g_quark_to_string(WEBKIT_POLICY_ERROR), WEBKIT_POLICY_ERROR_FRAME_LOAD_INTERRUPTED_BY_POLICY_CHANGE,
-                         request.url().string(), _("Frame load was interrupted"));
+    return WebCore::interruptedForPolicyChangeError(request);
 }
 
 ResourceError FrameLoaderClient::cannotShowMIMETypeError(const ResourceResponse& response)
 {
-    return ResourceError(g_quark_to_string(WEBKIT_POLICY_ERROR), WEBKIT_POLICY_ERROR_CANNOT_SHOW_MIME_TYPE,
-                         response.url().string(), _("Content with the specified MIME type cannot be shown"));
+    return WebCore::cannotShowMIMETypeError(response);
 }
 
 ResourceError FrameLoaderClient::fileDoesNotExistError(const ResourceResponse& response)
 {
-    return ResourceError(g_quark_to_string(WEBKIT_NETWORK_ERROR), WEBKIT_NETWORK_ERROR_FILE_DOES_NOT_EXIST,
-                         response.url().string(), _("File does not exist"));
+    return WebCore::fileDoesNotExistError(response);
 }
 
 ResourceError FrameLoaderClient::pluginWillHandleLoadError(const ResourceResponse& response)
 {
-    return ResourceError(g_quark_to_string(WEBKIT_PLUGIN_ERROR), WEBKIT_PLUGIN_ERROR_WILL_HANDLE_LOAD,
-                         response.url().string(), _("Plugin will handle load"));
+    return WebCore::pluginWillHandleLoadError(response);
 }
 
 bool FrameLoaderClient::shouldFallBack(const ResourceError& error)
